@@ -1,21 +1,25 @@
 # -*- coding: utf-8 -*-
+"""The main Process module"""
 
 from __future__ import absolute_import
 import abc
 import contextlib
 import functools
-from future.utils import with_metaclass, raise_
-from pika.exceptions import ConnectionClosed
 import copy
 import logging
 import time
 import sys
 import threading
+import uuid
+import six
+
+from pika.exceptions import ConnectionClosed
 from tornado import concurrent, gen
 import tornado.stack_context
-import uuid
 import yaml
 
+import kiwipy
+
 from .process_listener import ProcessListener
 from .process_spec import ProcessSpec
 from .utils import protected
@@ -34,6 +38,8 @@ from . import process_states
 from . import ports
 from . import utils
 
+# pylint: disable=too-many-lines
+
 __all__ = ['Process', 'ProcessSpec', 'BundleKeys', 'TransitionFailed']
 
 _LOGGER = logging.getLogger(__name__)
@@ -45,13 +51,14 @@ class BundleKeys(object):
 
     See :func:`save_instance_state` and :func:`load_instance_state`.
     """
+    # pylint: disable=too-few-public-methods
     INPUTS_RAW = 'INPUTS_RAW'
     INPUTS_PARSED = 'INPUTS_PARSED'
     OUTPUTS = 'OUTPUTS'
 
 
 # Use thread-local storage for the stack
-_thread_local = threading.local()
+_thread_local = threading.local()  # pylint: disable=invalid-name
 
 
 def _process_stack():
@@ -73,8 +80,9 @@ class ProcessStateMachineMeta(abc.ABCMeta, state_machine.StateMachineMeta):
 yaml.representer.Representer.add_representer(ProcessStateMachineMeta, yaml.representer.Representer.represent_name)
 
 
+@six.add_metaclass(ProcessStateMachineMeta)
 @persistence.auto_persist('_pid', '_CREATION_TIME', '_future', '_paused', '_status', '_pre_paused_status')
-class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.Savable)):
+class Process(StateMachine, persistence.Savable):
     """
     The Process class is the base for any unit of work in plumpy.
 
@@ -115,8 +123,6 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
     # Static class stuff ######################
     _spec_type = ProcessSpec
     # Default placeholders, will be populated in init()
-    _waiting_callbacks = None
-    _terminated_callbacks = None
     _stepping = False
     _pausing = None  # type: futures.Future
     _paused = None
@@ -127,8 +133,8 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
     def current(cls):
         if _process_stack():
             return _process_stack()[-1]
-        else:
-            return None
+
+        return None
 
     @classmethod
     def get_states(cls):
@@ -169,7 +175,7 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
         return cls.__name__
 
     @classmethod
-    def define(cls, spec):
+    def define(cls, _spec):
         cls.__called = True
 
     @classmethod
@@ -247,19 +253,21 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
         self._future = persistence.SavableFuture()
         self.__event_helper = utils.EventHelper(ProcessListener)
         self._logger = logger
-        if communicator is None:
-            self._communicator = None
-        else:
-            self._communicator = communications.CommunicatorWrapper(communicator, self._loop)
+        self._communicator = communicator
 
     @base.super_check
     def init(self):
         """ Any common initialisation stuff after create or load goes here """
-        self._waiting_callbacks = []
-        self._terminated_callbacks = []
-
         if self._communicator is not None:
-            self._communicator.add_rpc_subscriber(self.message_receive, identifier=str(self.pid))
+            try:
+                self._communicator.add_rpc_subscriber(self.message_receive, identifier=str(self.pid))
+            except kiwipy.TimeoutError:
+                self.logger.exception("Process<{}> failed to register as an RPC subscriber".format(
+                    self.pid))
+            try:
+                self._communicator.add_broadcast_subscriber(self.broadcast_receive)
+            except kiwipy.TimeoutError:
+                self.logger.exception("Process<{}> failed to register as a broadcast subscriber".format(self.pid))
 
         if not self._future.done():
 
@@ -324,8 +332,8 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
         """
         if self._logger is not None:
             return self._logger
-        else:
-            return _LOGGER
+
+        return _LOGGER
 
     @property
     def status(self):
@@ -344,7 +352,7 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
     def launch(self, process_class, inputs=None, pid=None, logger=None):
         process = process_class(
             inputs=inputs, pid=pid, logger=logger, loop=self.loop(), communicator=self._communicator)
-        self.call_soon_external(process.step_until_terminated)
+        self.create_background_task(process.step_until_terminated)
         return process
 
     # region State introspection methods
@@ -362,12 +370,12 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
         """
         if isinstance(self._state, process_states.Finished):
             return self._state.result
-        elif isinstance(self._state, process_states.Killed):
+        if isinstance(self._state, process_states.Killed):
             raise exceptions.KilledError()
-        elif isinstance(self._state, process_states.Excepted):
+        if isinstance(self._state, process_states.Excepted):
             raise self._state.exception
-        else:
-            raise exceptions.InvalidStateError
+
+        raise exceptions.InvalidStateError
 
     def successful(self):
         """
@@ -385,14 +393,14 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
     def killed_msg(self):
         if isinstance(self._state, process_states.Killed):
             return self._state.msg
-        else:
-            raise exceptions.InvalidStateError('Has not been killed')
+
+        raise exceptions.InvalidStateError('Has not been killed')
 
     def exception(self):
         if isinstance(self._state, process_states.Excepted):
             return self._state.exception
-        else:
-            return None
+
+        return None
 
     def done(self):
         """
@@ -419,14 +427,17 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
         self._loop.add_callback(handle.run)
         return handle
 
-    def call_soon_external(self, callback, *args, **kwargs):
+    def create_background_task(self, callback):
         """
-        Schedule a callback to an external method.  If there is an
-        exception in the callback it will not cause the process to fail.
-        """
-        self._loop.add_callback(callback, *args, **kwargs)
+        Create a task that corresponds to a callback scheduled on our event loop
 
-    def callback_excepted(self, callback, exception, trace):
+        :param callback: the callback to schedule, can be a function or coroutine
+        :return: a future corresponding to the result of this task
+        :rtype: :class:`plumpy.Future`
+        """
+        return futures.create_task(callback, loop=self._loop)
+
+    def callback_excepted(self, _callback, exception, trace):
         if self.state != process_states.ProcessState.EXCEPTED:
             self.fail(exception, trace)
 
@@ -447,18 +458,18 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
             _process_stack().pop()
 
     @gen.coroutine
-    def _run_task(self, fn, *args, **kwargs):
+    def _run_task(self, callback, *args, **kwargs):
         """
         This method should be used to run all Process related functions and coroutines.
         If there is an exception the process will enter the EXCEPTED state.
 
-        :param fn: A function or coroutine
+        :param callback: A function or coroutine
         :param args: Optional positional arguments passed to fn
         :param kwargs:  Optional keyword arguments passed to fn
         :return: The value as returned by fn
         """
         # Make sure execute is a coroutine
-        coro = utils.ensure_coroutine(fn)
+        coro = utils.ensure_coroutine(callback)
         result = yield tornado.stack_context.run_with_stack_context(
             tornado.stack_context.StackContext(self._process_scope), functools.partial(coro, *args, **kwargs))
         raise gen.Return(result)
@@ -509,14 +520,13 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
 
         self._state = self.recreate_state(saved_state['_state'])
 
-        if 'communicator' in load_context and load_context.communicator is not None:
-            self._communicator = communications.CommunicatorWrapper(load_context.communicator, self._loop)
+        if 'communicator' in load_context:
+            self._communicator = load_context.communicator
 
         if 'logger' in load_context:
             self._logger = load_context.logger
 
-        # Need to call this here as things downstream may rely on us having the
-        # runtime variable above
+        # Need to call this here as things downstream may rely on us having the runtime variable above
         super(Process, self).load_instance_state(saved_state, load_context)
 
         # Inputs/outputs
@@ -551,23 +561,11 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
     def set_logger(self, logger):
         self._logger = logger
 
-    # region Events
-
     @protected
     def log_with_pid(self, level, msg):
-        self.logger.log(level, "{}: {}".format(self.pid, msg))
+        self.logger.log(level, "%s: %s", self.pid, msg)
 
-    def add_on_waiting_callback(self, callback):
-        self._waiting_callbacks.append(callback)
-
-    def remove_on_waiting_callback(self, callback):
-        self._waiting_callbacks.remove(callback)
-
-    def add_on_terminated_callback(self, callback):
-        self._terminated_callbacks.append(callback)
-
-    def remove_on_terminated_callback(self, callback):
-        self._terminated_callbacks.remove(callback)
+    # region Events
 
     def on_entering(self, state):
         # Map these onto direct functions that the subclass can implement
@@ -605,8 +603,8 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
                 self._communicator.broadcast_send(
                     body=None, sender=self.pid, subject='state_changed.{}.{}'.format(from_label, self.state.value))
             except ConnectionClosed:
-                self.logger.info('no connection available to broadcast state change from {} to {}'.format(
-                    from_label, self.state.value))
+                self.logger.info('no connection available to broadcast state change from %s to %s', from_label,
+                                 self.state.value)
 
     def on_exiting(self):
         state = self.state
@@ -663,8 +661,6 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
     def on_waiting(self):
         """ Entered the WAITING state """
         self._fire_event(ProcessListener.on_process_waiting)
-        for cb in self._waiting_callbacks:
-            cb(self)
 
     @super_check
     def on_pausing(self, msg=None):
@@ -732,52 +728,100 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
         self._killing = None
         self._fire_event(ProcessListener.on_process_killed, self.killed_msg())
 
-    def on_terminated(self):
-        super(Process, self).on_terminated()
-        for cb in self._terminated_callbacks:
-            self.call_soon_external(cb, self)
-
     def _fire_event(self, evt, *args, **kwargs):
         self.__event_helper.fire_event(evt, self, *args, **kwargs)
 
     # endregion
 
-    @gen.coroutine
+    # region Communication
+
     def message_receive(self, _comm, msg):
+        """
+        Coroutine called when the process receives a message from the communicator
+
+        :param _comm: the communicator that sent the message
+        :type _comm: :class:`kiwipy.Communicator`
+        :param msg: the message
+        :return: the outcome of processing the message, the return value will be sent back as a response to the sender
+        """
+        self.logger.debug("RPC message '%s' received with communicator '%s'", msg, _comm)
+
         intent = msg[process_comms.INTENT_KEY]
 
         if intent == process_comms.Intent.PLAY:
-            result = self.play()
-        elif intent == process_comms.Intent.PAUSE:
-            result = self.pause(msg=msg.get(process_comms.MESSAGE_KEY, None))
-        elif intent == process_comms.Intent.KILL:
-            result = self.kill(msg=msg.get(process_comms.MESSAGE_KEY, None))
-        elif intent == process_comms.Intent.STATUS:
+            return self._schedule_rpc(self.play)
+        if intent == process_comms.Intent.PAUSE:
+            return self._schedule_rpc(self.pause, msg=msg.get(process_comms.MESSAGE_KEY, None))
+        if intent == process_comms.Intent.KILL:
+            return self._schedule_rpc(self.kill, msg=msg.get(process_comms.MESSAGE_KEY, None))
+        if intent == process_comms.Intent.STATUS:
             status_info = {}
             self.get_status_info(status_info)
-            result = status_info
-        else:
-            raise RuntimeError("Unknown intent")
+            return status_info
 
-        if concurrent.is_future(result):
-            # Wait for the process to actually finish
-            result = yield result
+        # Didn't match any known intents
+        raise RuntimeError("Unknown intent")
 
-        raise gen.Return(result)
+    def broadcast_receive(self, _comm, body, sender, subject, correlation_id):
+        """
+        Coroutine called when the process receives a message from the communicator
+
+        :param _comm: the communicator that sent the message
+        :type _comm: :class:`kiwipy.Communicator`
+        :param msg: the message
+        """
+        self.logger.debug("Broadcast message '%s' received with communicator '%s'", body, _comm)
+
+        # If we get a message we recognise then action it, otherwise ignore
+        if subject == process_comms.Intent.PLAY:
+            return self._schedule_rpc(self.play)
+        if subject == process_comms.Intent.PAUSE:
+            return self._schedule_rpc(self.pause, msg=body)
+        if subject == process_comms.Intent.KILL:
+            return self._schedule_rpc(self.kill, msg=body)
+
+    def _schedule_rpc(self, callback, *args, **kwargs):
+        """
+        Schedule a call to a callback as a result of an RPC communication call, this will return
+        a future that resolves to the final result (even after one or more layer of futures being
+        returned) of the callback.
+
+        :param callback: the callback function or coroutine
+        :param args: the positional arguments to the callback
+        :param kwargs: the keyword arguments to the callback
+        :return: a kiwi future that resolves to the outcome of the callback
+        :rtype: :class:`kiwipy.Future`
+        """
+
+        @gen.coroutine
+        def run_callback():
+            result = yield gen.coroutine(callback)(*args, **kwargs)
+            while concurrent.is_future(result):
+                result = yield result
+            raise gen.Return(result)
+
+        # Schedule the task and give back a kiwi future
+        task = self.create_background_task(run_callback)
+        return communications.plum_to_kiwi_future(task)
+
+    # endregion
 
     def close(self):
         """
         Remove all the RPC subscribers from the communicator tied to the process
         """
         if self._communicator is not None:
-            self._communicator.remove_rpc_subscriber(str(self.pid))
+            try:
+                self._communicator.remove_rpc_subscriber(str(self.pid))
+            except (kiwipy.TimeoutError, ValueError):
+                self.logger.exception("Process<{}> failed to remove itself as an RPC subscriber".format(self.pid))
 
     # region State related methods
 
-    def transition_excepted(self, initial_state, final_state, exception, trace):
+    def transition_excepted(self, _initial_state, final_state, exception, trace):
         # If we are creating, then reraise instead of failing.
         if final_state == process_states.ProcessState.CREATED:
-            raise_(type(exception), exception, trace)
+            six.reraise(type(exception), exception, trace)
         else:
             self.transition_to(process_states.ProcessState.EXCEPTED, exception, trace)
 
@@ -809,8 +853,8 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
             # Try to interrupt the state
             self._state.interrupt(interrupt_exception)
             return self._interrupt_action
-        else:
-            return self._do_pause(msg)
+
+        return self._do_pause(msg)
 
     def _do_pause(self, state_msg, next_state=None):
         """ Carry out the pause procedure, optionally transitioning to the next state first"""
@@ -836,7 +880,7 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
             do_pause = functools.partial(self._do_pause, str(exception))
             return futures.CancellableAction(do_pause, cookie=exception)
 
-        elif isinstance(exception, process_states.KillInterruption):
+        if isinstance(exception, process_states.KillInterruption):
 
             def do_kill(_next_state):
                 try:
@@ -847,8 +891,8 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
                     self._killing = None
 
             return futures.CancellableAction(do_kill, cookie=exception)
-        else:
-            raise ValueError("Got unknown interruption type '{}'".format(type(exception)))
+
+        raise ValueError("Got unknown interruption type '{}'".format(type(exception)))
 
     def _set_interrupt_action(self, new_action):
         """
@@ -883,9 +927,7 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
 
     @event(from_states=(process_states.Running, process_states.Waiting))
     def resume(self, *args):
-        """
-        Start running the process again
-        """
+        """Start running the process again"""
         return self._state.resume(*args)
 
     @event(to_states=process_states.Excepted)
@@ -923,9 +965,9 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
             self._killing = self._interrupt_action
             self._state.interrupt(interrupt_exception)
             return self._interrupt_action
-        else:
-            self.transition_to(process_states.ProcessState.KILLED, msg)
-            return True
+
+        self.transition_to(process_states.ProcessState.KILLED, msg)
+        return True
 
         # endregion
 
@@ -1090,6 +1132,7 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
         :param inputs: A mapping of the inputs as passed to the process
         :return: The encoded inputs
         """
+        # pylint: disable=no-self-use
         return copy.deepcopy(inputs)
 
     @protected
@@ -1102,6 +1145,7 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
         :param encoded:
         :return: The decoded input args
         """
+        # pylint: disable=no-self-use
         return copy.deepcopy(encoded)
 
     def get_status_info(self, out_status_info):
@@ -1113,10 +1157,6 @@ class Process(with_metaclass(ProcessStateMachineMeta, StateMachine, persistence.
             'state_info': str(self._state)
         })
 
-    # region State entry/exit events
-
-    # endregion
-
     def _check_inputs(self, inputs):
         # Check the inputs meet the requirements
         valid, msg = self.spec().validate_inputs(inputs)
