@@ -1,15 +1,18 @@
+from __future__ import absolute_import
 from functools import partial
 import shutil
 import tempfile
-from tornado import testing
 import unittest
 import uuid
+import shortuuid
 
 from kiwipy import rmq
+from six.moves import range
+from tornado import testing, ioloop
 
 import plumpy.test_utils
-
 from plumpy import communications, process_comms, test_utils
+from ..utils import AsyncTestCase
 
 try:
     import pika
@@ -18,24 +21,91 @@ except ImportError:
 
 AWAIT_TIMEOUT = testing.get_async_test_timeout()
 
+# pylint: disable=missing-docstring
+
+
+class CommunicatorTestCase(AsyncTestCase):
 
-@unittest.skipIf(not pika, "Requires pika library and RabbitMQ")
-class TestTaskActions(testing.AsyncTestCase):
     def setUp(self):
-        super(TestTaskActions, self).setUp()
-        self.loop = self.io_loop
-
-        exchange_name = "{}.{}".format(self.__class__.__name__, uuid.uuid4())
-        queue_name = "{}.{}.tasks".format(self.__class__.__name__, uuid.uuid4())
+        super(CommunicatorTestCase, self).setUp()
+        message_exchange = "{}.{}".format(self.__class__.__name__, shortuuid.uuid())
+        task_exchange = "{}.{}".format(self.__class__.__name__, shortuuid.uuid())
+        queue_name = "{}.{}.tasks".format(self.__class__.__name__, shortuuid.uuid())
 
         self.rmq_communicator = rmq.connect(
             connection_params={'url': 'amqp://guest:guest@localhost:5672/'},
-            message_exchange=exchange_name,
+            message_exchange=message_exchange,
+            task_exchange=task_exchange,
             task_queue=queue_name,
-            testing_mode=True
-        )
-        self.communicator = communications.CommunicatorWrapper(self.rmq_communicator, self.loop)
+            testing_mode=True)
+        self.communicator = communications.LoopCommunicator(self.rmq_communicator, self.loop)
 
+    def tearDown(self):
+        # Close the connector before calling super because it will close the loop
+        self.rmq_communicator.stop()
+        super(CommunicatorTestCase, self).tearDown()
+
+
+@unittest.skipIf(not pika, "Requires pika library and RabbitMQ")
+class TestLoopCommunicator(CommunicatorTestCase):
+    """Make sure the loop communicator is working as expected"""
+
+    @testing.gen_test
+    def test_broadcast(self):
+        BROADCAST = {'body': 'present', 'sender': 'Martin', 'subject': 'sup', 'correlation_id': 420}
+        broadcast_future = plumpy.Future()
+
+        def get_broadcast(_comm, body, sender, subject, correlation_id):
+            self.assertEqual(self.loop, ioloop.IOLoop.current())
+            broadcast_future.set_result({
+                'body': body,
+                'sender': sender,
+                'subject': subject,
+                'correlation_id': correlation_id
+            })
+
+        self.communicator.add_broadcast_subscriber(get_broadcast)
+        self.communicator.broadcast_send(**BROADCAST)
+
+        result = yield broadcast_future
+        self.assertDictEqual(BROADCAST, result)
+
+    @testing.gen_test
+    def test_rpc(self):
+        MSG = 'rpc this'
+        rpc_future = plumpy.Future()
+
+        def get_rpc(_comm, msg):
+            self.assertEqual(self.loop, ioloop.IOLoop.current())
+            rpc_future.set_result(msg)
+
+        self.communicator.add_rpc_subscriber(get_rpc, 'rpc')
+        self.communicator.rpc_send('rpc', MSG)
+
+        result = yield rpc_future
+        self.assertEqual(MSG, result)
+
+    @testing.gen_test
+    def test_task(self):
+        TASK = 'task this'
+        task_future = plumpy.Future()
+
+        def get_task(_comm, msg):
+            self.assertEqual(self.loop, ioloop.IOLoop.current())
+            task_future.set_result(msg)
+
+        self.communicator.add_task_subscriber(get_task)
+        self.communicator.task_send(TASK)
+
+        result = yield task_future
+        self.assertEqual(TASK, result)
+
+
+@unittest.skipIf(not pika, "Requires pika library and RabbitMQ")
+class TestTaskActions(CommunicatorTestCase):
+
+    def setUp(self):
+        super(TestTaskActions, self).setUp()
         self._tmppath = tempfile.mkdtemp()
         self.persister = plumpy.PicklePersister(self._tmppath)
         # Add the process launcher
@@ -45,8 +115,6 @@ class TestTaskActions(testing.AsyncTestCase):
 
     def tearDown(self):
         # Close the connector before calling super because it will
-        # close the loop
-        self.rmq_communicator.stop()
         super(TestTaskActions, self).tearDown()
         shutil.rmtree(self._tmppath)
 
